package com.logicbig.example;

import javax.persistence.*;
import java.util.HashMap;
import java.util.Map;

public class ExampleMain2 {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEntity();
            findEntity();
        } finally {
            entityManagerFactory.close();
        }
    }

    public static void persistEntity() {
        TradingModule module = getTradingModule();
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(module);
        em.getTransaction().commit();
        em.close();
    }

    private static void findEntity() {
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityGraph graph = em.getEntityGraph("graph.module.projects.contractors");
        Map<String, Object> properties = new HashMap<>();
        properties.put("javax.persistence.fetchgraph", graph);
        TradingModule module = em.find(TradingModule.class, 1, properties);
        em.close();
        printInitializeState(module);
    }

    private static void printInitializeState(TradingModule module) {
        PersistenceUtil pu = entityManagerFactory.getPersistenceUnitUtil();
        printState("TradingModule", pu.isLoaded(module));
        printState("TradingModule.name", pu.isLoaded(module, "name"));
        printState("TradingModule.projects", pu.isLoaded(module, "projects"));
        if (pu.isLoaded(module, "projects")) {
            for (Project project : module.getProjects()) {
                printState("Project", pu.isLoaded(project));
                printState("Project.name", pu.isLoaded(module, "project.name"));
                printState("Projects.contractors", pu.isLoaded(module, "project.contractors"));
            }
        }
    }

    private static void printState(String msg, boolean loaded) {
        System.out.printf("%25s loaded: %s%n", msg, loaded);
    }

    public static TradingModule getTradingModule() {
        Project project = new Project();
        project.setName("Price/Yield Calculator");
        project.addContractor(new Contractor("Margaret", "Project Manager"));
        project.addContractor(new Contractor("Steven", "Developer"));
        project.addContractor(new Contractor("Claudie", "Business Analyst"));

        TradingModule module = new TradingModule();
        module.setName("Fixed Income");
        module.addProject(project);
        return module;
    }
}